% Pattern recognition project
% Chen bladder cancer

% Identify columns of data...
normalIndex=[103:124]; nni=length(normalIndex);
primaryTumorIndexAll=[1:6,8:102,125,126]; npta=length(primaryTumorIndexAll);

xraw = load('c:\user_data\csuf\project\bladderCancer\imputed.txt');
xraw=xraw(:,[primaryTumorIndexAll normalIndex]);
primaryTumorIndexAll=1:npta;

% Remove means
nrx=size(xraw,1);
for i=1:nrx
    xraw(i,:)=xraw(i,:)-mean(xraw(i,:));
end

ntests=100;
ntestTumor=20;
nPOD=npta-ntestTumor;
PODsamples=zeros(npta,ntests);
for k=1:ntests
    
    fprintf('case # %d\n',k);
    
    clear testTumorIndex primaryTumorIndex
    testTumorIndex=unique(floor(npta*rand(1,ntestTumor)+1));
    while (length(testTumorIndex)<ntestTumor) | (max(testTumorIndex)>npta)
        testTumorIndex=unique(floor(npta*rand(1,ntestTumor)+1));
        fprintf('Iteration required in random loop...\n');
    end
    testTumorIndex=reshape(testTumorIndex,1,length(testTumorIndex));
    primaryTumorIndex=setxor(1:npta,testTumorIndex);
    primaryTumorIndex=reshape(primaryTumorIndex,1,length(primaryTumorIndex));
    
%    x=xraw(:,primaryTumorIndex);
    clear x
    for i=1:length(primaryTumorIndex)
        x(:,i)=xraw(:,primaryTumorIndex(i));
        PODsamples(primaryTumorIndex(i),k)=1;
    end

    % Generate covariance
    ns = length(primaryTumorIndex);
    nn=length(normalIndex);
    nt=length(testTumorIndex);

    gridSize = size(x,1);
    theta = zeros(ns);
    for i=1:ns
        vi = x(:,i);
        for j=i:ns
            vj=x(:,j);
            theta(i,j) = vi'*vj/gridSize/ns;
            theta(j,i) = theta(i,j);
        end
    end

    % Compute dominant eigenvalue/eigenvector
    [u,lam] = eig(theta);
    [lam, ind] = sort(-diag(lam));
    lam=-lam;
    u=u(:,ind');

    % Normalize so that first eigenvector has unit length
    for i=1:ns
        u(:,i)=u(:,i)/sum(abs(u(:,i)));
    end

    % Normalized eigenvectors...
    for i=1:ns
        phi(:,i)=zeros(gridSize,1);
        for j=1:ns
            phi(:,i)=phi(:,i)+u(j,i)*x(:,j);
        end
        phi(:,i)=phi(:,i)/norm(phi(:,i));
        normPhi(i)=phi(:,i)'*phi(:,i);
    end
    
    % Determine sign for dominant eigenvector...select sign so that the sum
    % of the projections of the tumor samples used for the POD is greater
    % than 1.
    for j=1:ns
        sumP1(j)=0.;
        for i=1:ns
            sumP1(j)=sumP1(j)+sum(x(:,i)'*phi(:,j)/normPhi(j));
        end
        if sumP1(j) < 0, phi(:,j)=-phi(:,j); end
            
        % Projections...
        for i=1:size(xraw,2)
            projMatA(i,j,k) = xraw(:,i)'*phi(:,j)/normPhi(j);
        end
    end
end

% for plotting, need to duplicate the last row and column so they will be
% plotted
for j=1:2
    projMat=squeeze(projMatA(:,j,:));
	[nr,nc]=size(projMat);
	outData(j).pltProjMat=projMat;
	outData(j).pltProjMat(nr+1,:)=outData(j).pltProjMat(nr,:);
	outData(j).pltProjMat(:,nc+1)=outData(j).pltProjMat(:,nc);
	[nr,nc]=size(PODsamples);
	outData(j).pltPODsamples=PODsamples;
	outData(j).pltPODsamples(nr+1,:)=outData(j).pltPODsamples(nr,:);
	outData(j).pltPODsamples(:,nc+1)=outData(j).pltPODsamples(:,nc);
end

figure(1)
pcolor(outData(1).pltProjMat)
colorbar
xlabel('Case Number')
set(gca,'XTick',10:10:100);
ylabel('Sample number')
title('Projections on Principal Component')
print -dtiff bladder_projection1_random

figure(2)
pcolor(outData(2).pltProjMat)
colorbar
xlabel('Case Number')
set(gca,'XTick',10:10:100);
ylabel('Sample number')
title('Projections on 2nd Principal Component')
print -dtiff bladder_projection2_random

% Statistics for projections
for i=1:2
	tumorMean(i,:)=mean(outData(i).pltProjMat(1:npta,:));
	tumorStd(i,:)=std(outData(i).pltProjMat(1:npta,:));
	normalMean(i,:)=mean(outData(i).pltProjMat((npta+1):size(outData(i).pltProjMat,1),:));
	normalStd(i,:)=std(outData(i).pltProjMat((npta+1):size(outData(i).pltProjMat,1),:));
end

save chenBladderSave

% Investigate whether or not the normal distribution assumption is valid
% Just do for representative case
for k=1:1
	for j=1:ntests
		caseNo=j;
		znormal=sort((outData(k).pltProjMat((npta+1):size(outData(k).pltProjMat,1),caseNo)-mean(normalMean(k,:)))/mean(normalStd(k,:)));
		ztumor=sort((outData(k).pltProjMat(1:npta,caseNo)-mean(tumorMean(k,:)))/mean(tumorStd(k,:)));
		nmax=length(znormal);
		for i=1:nmax
           alphaLevel=i/nmax;
           zalpha_normal(i)=norminv(alphaLevel,0,1);
		end
		nmax=length(ztumor);
		for i=1:nmax
           alphaLevel=i/nmax;
           zalpha_tumor(i)=norminv(alphaLevel,0,1);
		end
		figure(3+k)
        subplot(2,1,1)
		plot(zalpha_tumor,ztumor,'k*',zalpha_tumor,zalpha_tumor,'k:',zalpha_tumor,3+zalpha_tumor,'k:',zalpha_tumor,zalpha_tumor-3,'k:')
        title(['Chen Bladder Cancer - Tumor Samples - Component ' sprintf('%d', k)])
        xlabel('Standard Normal Percentile')
        ylabel('Normalized projection')
        hold on
		subplot(2,1,2)
		plot(zalpha_normal,znormal,'k*',zalpha_normal,zalpha_normal,'k:',zalpha_normal,3+zalpha_normal,'k:',zalpha_normal,zalpha_normal-3,'k:')
        title('Normal Samples')
        xlabel('Standard Normal Percentile')
        ylabel('Normalized projection')
        hold on
	end
	subplot(2,1,1)
	hold off
	subplot(2,1,2)
	hold off
    if k==1
    	print -dtiff bladder_checkNormal1
    else
        print -dtiff bladder_checkNormal2
    end
    
    ctr=1:ntests;

	x=-80:0.1:80;
    ftumor=1/sqrt(2*pi)/mean(tumorStd(k,:))*exp(-(x-mean(tumorMean(k,:))).^2/(2*mean(tumorStd(k,:))^2));
	fnormal=1/sqrt(2*pi)/mean(normalStd(k,:))*exp(-(x-mean(normalMean(k,:))).^2/(2*mean(normalStd(k,:))^2));
	figure(5+k)
	plot(x,ftumor,'k-',x,fnormal,'r-.')
	legend('Tumor','Normal')
    title(['Chen Bladder Cancer Normal Distribution Curves - Component ' sprintf('%d',k)])
	xlabel('Projection Value')
	ylabel('Normal Probability Density Function')
    if k==1
    	print -dtiff bladder_normalDist1
    else
        print -dtiff bladder_normalDist2
    end
end

% Try using sum of the first two projections
sumProjMat=outData(1).pltProjMat + outData(2).pltProjMat;
sumTumorMean=mean(sumProjMat(1:npta,:));
sumTumorStd=std(sumProjMat(1:npta,:));
sumNormalMean=mean(sumProjMat((npta+1):size(sumProjMat,1),:));
sumNormalStd=std(sumProjMat((npta+1):size(sumProjMat,1),:));

% Check normal distribution assumption
for j=1:ntests
	caseNo=j;
	znormal=sort((sumProjMat((npta+1):size(sumProjMat,1),caseNo)-mean(sumNormalMean))/mean(sumNormalStd));
	ztumor=sort((sumProjMat(1:npta,caseNo)-mean(sumTumorMean))/mean(sumTumorStd));
	nmax=length(znormal);
	for i=1:nmax
       alphaLevel=i/nmax;
       zalpha_normal(i)=norminv(alphaLevel,0,1);
	end
	nmax=length(ztumor);
	for i=1:nmax
       alphaLevel=i/nmax;
       zalpha_tumor(i)=norminv(alphaLevel,0,1);
	end
	figure(8)
    subplot(2,1,1)
	plot(zalpha_tumor,ztumor,'k*',zalpha_tumor,zalpha_tumor,'k:',zalpha_tumor,3+zalpha_tumor,'k:',zalpha_tumor,zalpha_tumor-3,'k:')
    title('Chen Bladder Cancer - Tumor Samples - Sum of Projections onto Components 1 and 2')
    xlabel('Standard Normal Percentile')
    ylabel('Normalized projection')
    hold on
	subplot(2,1,2)
	plot(zalpha_normal,znormal,'k*',zalpha_normal,zalpha_normal,'k:',zalpha_normal,3+zalpha_normal,'k:',zalpha_normal,zalpha_normal-3,'k:')
    title('Normal Samples')
    xlabel('Standard Normal Percentile')
    ylabel('Normalized projection')
    hold on
end
subplot(2,1,1)
hold off
subplot(2,1,2)
hold off
print -dtiff bladder_checkNormalSum

ftumor=1/sqrt(2*pi*mean(sumTumorStd))*exp(-(x-mean(sumTumorMean)).^2/(2*mean(sumTumorStd)^2));
fnormal=1/sqrt(2*pi*mean(sumNormalStd))*exp(-(x-mean(sumNormalMean)).^2/(2*mean(sumNormalStd)^2));
figure(9)
plot(x,ftumor,'k-',x,fnormal,'r-')
legend('Tumor','Normal')
title('Chen Bladder Cancer Normal Distribution Curves - Sum of Projections onto Components 1 and 2')
xlabel('Projection Value')
ylabel('Normal Probability Density Function')
print -dtiff bladder_normalDistSum

tumorCDF1 = normcdf(0,mean(tumorMean(1,:)),mean(tumorStd(1,:)))
tumorCDF2 = normcdf(0,mean(tumorMean(2,:)),mean(tumorStd(2,:)))
sumCDF = normcdf(0, mean(sumTumorMean), mean(sumTumorStd))

tMu=mean(tumorMean(1,:));
tSigma=mean(tumorStd(1,:));
nMu=mean(normalMean(1,:));
nSigma=mean(normalStd(1,:));
ftumor=normpdf(x,tMu,tSigma);
fnormal=normpdf(x,nMu,nSigma);
xp=-20:1:80;

%ft=1/sqrt(2*pi)/tSigma*exp(-(x-tMu).^2/(2*tSigma^2));
%fn=1/sqrt(2*pi)/nSigma*exp(-(x-nMu).^2/(2*nSigma^2));
% Check if cancer
ft=normpdf(xp,tMu,tSigma);
fn=normpdf(xp,nMu,nSigma);
fp=normpdf(xp,tMu+nMu,sqrt(tSigma^2+nSigma^2));
lrt=ft./((ft+fn));
% for i=1:length(xp)
%     if (xp(i) < -19)
%         if lrt(i)>.5*(ft(i)+fn(i))
%             lrt(i)=.5*(ft(i)+fn(i));
%         end
% 	end
%     %lrt(i)=1-fn(i)./(ft(i)+fn(i));
% end
%lrt=ft./fp;
figure(20)
plot(x,ftumor,'k-',x,fnormal,'r-')
hold on
figure(20)
[AX,H1,H2] = plotyy(x,ftumor,xp,lrt,'plot');
plotyy(x,ftumor,xp,lrt)
xlabel('Projection')
set(get(AX(1),'Ylabel'),'String','Distribution Function')
set(get(AX(2),'Ylabel'),'String','Prob of Cancer')
grid on
set(AX(1),'YTick',[0:.02:.1])
set(AX(2),'YTick',[0:.2:1])
set(AX(1),'YColor',[0 0 0])
set(AX(2),'YColor',[0 0 0])
title('Chen Bladder Cancer Normal Distribution Curves & Prob of Cancer')
hold off
print -dpng normDist_probCancer

figure(21)
subplot(2,1,1)
plot(x,ftumor,'k-',x,fnormal,'r-.')
title('Chen Bladder Cancer Normal Distribution Curves')
xlabel('Projection Value')
ylabel('Normal Distribution')
axis([-40 50 0 0.08])
legend('Cancer','Normal')
subplot(2,1,2)
ptumor=1-fnormal./(ftumor+fnormal)
plot(x(501:1601),ptumor(501:1601),'k-')
title('Chen Bladder Cancer - Probability of Cancer')
xlabel('Projection Value')
ylabel('Probability of Cancer')
axis([-40 50 0 1])
print -dpng normDist_probCancer2

xciMean=mean(outData(1).pltProjMat(1:npta,:));
xciStd=std(outData(1).pltProjMat(1:npta,:));
pltciLow=xciMean-xciStd*tinv(.025,npta-1)/sqrt(npta);
pltciHi=xciMean+xciStd*tinv(.025,npta-1)/sqrt(npta);
figure(22)
xcinormalMean=mean(outData(1).pltProjMat((npta+1):size(outData(1).pltProjMat,1),:));
xcinormalStd=std(outData(1).pltProjMat((npta+1):size(outData(1).pltProjMat,1),:));
pltcinLow=xcinormalMean-xcinormalStd*tinv(.025,npta-1)/sqrt(npta);
pltcinHi=xcinormalMean+xcinormalStd*tinv(.025,npta-1)/sqrt(npta);
plot(1:ntests,pltciLow(1:ntests),'r:',1:ntests,xciMean(1:ntests),'r-',1:ntests,pltciHi(1:ntests),'r:', ...
    1:ntests,pltcinLow(1:ntests),'k:',1:ntests,xcinormalMean(1:ntests),'k-',1:ntests,pltcinHi(1:ntests),'k:')
xlabel('Case Number')
ylabel('95% confidence intervals')
title('Chen Bladder Cancer - 95% Confidence Intervals for Means')
